% init_tinkpar1.m initializes parameters for a crude estimate of the
% Tinkertoy walker. 

% All code by Mariano Garcia (msg5@cornell.edu), 1998
% Some documentation of sorts and accompanying figures can
% be found in chaps 2 and 5 of my thesis, which can be downloaded from
% http://tam.cornell.edu/students/garcia/msghomepage.html
% or
% http://www.ccmr.cornell.edu/~ruinalab/
% also read the README.txt file

% These are not the most up-to-date parameters but the best file is
% on my PC, which is in a box somewhere. This file is OK though.
% It assumes that each leg of the walker is composed of two point-masses.
% One is on the outrigger and the other represents the rest of the leg.

% see the files 3D_diskfoot_frames.eps and my.mcgeer.config2.eps for
% illustrations of the parameters and variables.

clear
global g gam m R p23 p34 pc3 pc4 p4f Icm3 Icm4 h hspr hdamp sspr sdamp
format long

g=1;   % gravitational constant
h=0.35;  % hip width (normalized to leg length, which is 1)
gam=0.07;  % ramp slope
R=0.19;   % foot radius
m=[0 0 1 1]';  % m(1) and m(2) are fictitious links and are not used.
               % m(3) and m(4) are the mass of the stance and swing legs, resp.
p23=[R 0 0]';  % vector from ground contact to center of stance foot in stance leg
               % coordinates.
p34=[1-R 0 -h]'; % vector from center of stance foot to hip hinge (located
                 % at top of swing leg), in stance leg coordinates.
mo=0.66;  % outrigger mass on each leg
mr=0.34;  % rest of each leg

pcout=[0.14 0 1]'; % cm of outrigger only, origin at ctr of foot

angle=0; % outrigger rotation about z axis (parallel to hip).
         % realistic value is something like 
Rotz=[cos(angle) -sin(angle) 0; sin(angle) cos(angle) 0; 0 0 1];

sag = 0; %outrigger rotation about y axis; also don't know if this matters.
Roty=[cos(sag) 0 sin(sag); 0 1 0; -sin(sag) 0 cos(sag)];

pcout3=Rotz*pcout; % reset outrigger cm position according to sag and angle.
pcout3=Roty*pcout3;

pc3noout=[0.32 0 0]'; % cm of rest of walker in stance leg coordinates.

pc3=mo*pcout3+mr*pc3noout; % compute cm of whole walker.

pc4=[1-pc3(1)-R -pc3(2) -pc3(3) ]';  % compute cm of swing leg in swing leg
                                     % coordinates
p4f=[p34(1) 0 0]';  % vector from swing leg origin to center of swing foot.

% Compute the I matrix of the walker about its cm, based on two point masses.
% Use parallel axis theorem
Icm3noout=zeros(3,3);
diffr=-(pc3-pc3noout);
diffo=-(pc3-pcout3);

Icm3 = Icm3noout + mo*(diffo'*diffo*eye(3) - diffo*diffo') + ...
   mr*(diffr'*diffr*eye(3) - diffr*diffr');

Icm4=Icm3;
hspr = 0.0000;  % torsional hip spring stiffness
hdamp = 0.0000; % torsional hip damping
sspr = 0;  % torsional steering spring
sdamp = 0; % torsional steer damping (torque at contact patch)

% ICs for two cases. For simple forward integration with animation, add
% 0 0 to the end of the ICs, as with the first set below.
% For fixed-point finding, do not include 0 0 at the end. 
% ICs are heading angle, lean angle, stance pitch, swing pitch,
%         heading rate,  lean rate,  stance pitch rate, swing pitch rate.
% See figures in chap 2 of thesis for more clarification.

% Also search for SUBTLE POINT in int_walker_3D.m

%gam=0.07; %R=0.19 Icm3=tink w/ rot 0, hip=0.35, tol = 1e-10
y0=[0.09171869171274   1.56127421096639  -0.17493050240206   3.46597224175620 ...
-0.16949083860114  -0.01235967032945   0.45806348497766  -0.60422920671979 0 0 ];

%gam=0.07; %R=0.19 Icm3=tink w/ rot 0, hip=0.35, tol = 1e-10
y0=[0.09171869171274   1.56127421096639  -0.17493050240206   3.46597224175620 ...
-0.16949083860114  -0.01235967032945   0.45806348497766  -0.60422920671979 ];

%gam=0.07; %R=0.19 Icm3=tink w/ rot 0, hip=0.35 short
%y0=[0.09579009820011   1.56623840997369  -0.17083987834258   3.47127011465974 ...
%-0.16490550200253   0.03018638973522   0.74016372465624  -0.57770955646332 ];


t0=0;  % initial time, useless
tfinal=5;  % final time
tol=1e-10;  % integration tolerance. Usually I use 1e-10 if I want to
            % get numerical values for eigenvalues.

% Now run
% [tout,yout,step]=int_walker_3D(t0,tfinal,y0,tol);
% for forward integration with animation,
% then to plot mcgeer params:
% plot(tout,-yout(:,1),tout,-yout(:,2)+pi/2);
% plot(tout,yout(:,3),tout,yout(:,3)+yout(:,4)-pi);

% to find fixed points, run 
% it_3D_mcg

