function [tout,yout,step]=int_walker_3D(t0,tfinal,y0,tol);


% Does forward integration and animation for a 3D walker with
% 4 DOF. Assumes that relevant init file has been called.

global g gam m R p01 p23 p34 pc3 pc4 p4f Icm3 Icm4 hspr hdamp sspr sdamp
% don't put global h up here, since a different h is used in here!

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%       C.B. Moler, 3-25-87, 10-5-91, 6-3-93.
%       Copyright (c) 1984-93 by The MathWorks, Inc.
%       Modified for walking simulations
%       by Mariano Garcia (msg5@cornell.edu), 1998
%%%%%%%%%%%%%%%%%%%%%%%%%% Program begins  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% See SUBTLE POINT, below

% Initialization
% The Fehlberg coefficients:
beta  = [ [    1      0      0     0      0    0]/4
          [    3      9      0     0      0    0]/32
          [ 1932  -7200   7296     0      0    0]/2197
          [ 8341 -32832  29440  -845      0    0]/4104
          [-6080  41040 -28352  9295  -5643    0]/20520 ]';
gamma = [ [902880  0  3953664  3855735  -1371249  277020]/7618050
          [ -2090  0    22528    21970    -15048  -27360]/752400 ]';
pow = 1/5;  
f = zeros(length(y0),6);
t = t0;
w=1; % for counting walker steps
step(1,:)=y0;
hmax = (tfinal - t)/16;  
h = hmax/8; 
y = y0(:);
chunk = 128;  
tout = zeros(chunk,1);
yout = zeros(chunk,length(y));
k=1; % the usual MATLAB k to count time steps
tout(k) = t;
yout(k,:) = y.';
tau = tol * max(norm(y, 'inf'), 1);
clf reset;
axis([-0.5 0.5 -1 0.5 0 1]);
axis equal;

hf1=0; % hf1 is height of swing foot above ramp surface
hfdot1=1; % hfdot1 is the vertical velocity of the swing foot

init_graphics_3D(y);  % cal the graphics initialization routine

%%%%%%%%%%%%%%%%%%%%%  Main Loop Begins  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% See SUBTLE POINT, below
while t<tfinal

p01 = [0 0 0]'; % Put the origin of the fixed frame initially at the
                % foot contact point. For graphics only, not relevant to
                % dynamics.

  while  (hfdot1>0) & t < tfinal  % While the swing foot is on the way up
                                  % and t < tfinal
 % Compute the slopes
   junk = yderivs_3D(t,y);
   f(:,1) = junk(1:10);   % to track foot contact point: f(:,1) = junk(1:10);
   hf1=junk(11);          
   hfdot1 = junk(12);     % y(9) and y(10) are x and y of frame 0 (I think).
   for j = 1:5
    junk = yderivs_3D(t,y+h*f*beta(:,j));
    f(:,j+1) = junk(1:10); 
   end % j = 1:5 loop

 % Estimate the error and the acceptable error; a standard MATLAB thing.
   delta = norm(h*f*gamma(:,2),'inf');
   tau = tol*max(norm(y,'inf'),1.0);

 % Update the solution only if the error is acceptable; more MATLAB wizardry
   if delta <= tau 
      t = t + h;
      incr = h*f*gamma(:,1);
      y = y + incr;
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end
      tout(k) = t;
      yout(k,:) = y';
     dographics_3D(y);  % update the graphics
   end;  % delta <= tau loop

        % Update the step size
     if delta ~= 0.0
      h = min(hmax, 0.8*h*(tau/delta)^pow);
     end % delta ~= 0.0 loop

  end;  % (hfdot1>0) & t < tfinal loop

% SUBTLE POINT: to duplicate McGeer's fixed point, one needs to let the
% swing foot pass underground slightly. The fixed points that are in
% init_tinkpar1.m also need this allowance. The more up-to-date fixed
% points that are on my personal PC (in transit to CA) do not need this
% allowance and will not work with this integrator as is.
% The following loops allow for this passing underground.
% If this is not desired, get rid of the first two if-then loops.
% I have another version of this integrator with these two
% loops removed.

  while  ((hf1+h*hfdot1<0) | hfdot1 < 0) & t < tfinal
   % This particular loop is while the foot is on the way down
   % or below ground
   % this is a duplication of the loop above

   % Sometimes this integrator will fail because the swing foot never comes
   % back above the ground.  Have to watch out for that.

   junk = yderivs_3D(t,y); % get the slopes from the deriv file.
   f(:,1) = junk(1:10);  % to track foot contact point: f(:,1) = junk(1:10);
                         % y(9) and y(10) are x and y of frame 0 (I think).
   hf1=junk(11);  % height of swing foot bottom
   hfdot1 = junk(12);  % vertical velocity of swing foot bottom
   for j = 1:5
    junk = yderivs_3D(t,y+h*f*beta(:,j));
    f(:,j+1) = junk(1:10);
   end %j = 1:5 loop

   % Estimate the error and the acceptable error; a standard MATLAB thing.
   delta = norm(h*f*gamma(:,2),'inf');
   tau = tol*max(norm(y,'inf'),1.0);

        % Update the solution only if the error is acceptable
   if delta <= tau
      t = t + h;
      incr = h*f*gamma(:,1);
      y = y + incr;
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end % k > length(tout) loop
      tout(k) = t;
      yout(k,:) = y';
      dographics_3D(y); % update the graphics
   end; % delta <= tau loop

        % Update the step size

     if delta ~= 0.0
      h = min(hmax, 0.8*h*(tau/delta)^pow);
     end % delta ~= 0.0 loop

   end; %((hf1+h*hfdot1<0) | hfdot1 < 0) & t < tfinal loop

  while  (hf1+h*hfdot1>0) & t < tfinal
   % Now the foot is back above ground.
   % Heelstrike will occur when the swing foot hits the ground.

   junk = yderivs_3D(t,y); % get the slopes from the deriv file.
   f(:,1) = junk(1:10); % to track foot contact point: f(:,1) = junk(1:10);
                        % y(9) and y(10) are x and y of frame 0 (I think).
   hf1=junk(11); % height of swing foot bottom
   hfdot1 = junk(12); % vertical velocity of swing foot bottom
   for j = 1:5
    junk = yderivs_3D(t,y+h*f*beta(:,j));
    f(:,j+1) = junk(1:10);
   end %j = 1:5 loop

   % Estimate the error and the acceptable error; a standard MATLAB thing.
   delta = norm(h*f*gamma(:,2),'inf');
   tau = tol*max(norm(y,'inf'),1.0);

        % Update the solution only if the error is acceptable
   if delta <= tau
      t = t + h;
      incr = h*f*gamma(:,1);
      y = y + incr;
      k = k+1;
      if k > length(tout)
         tout = [tout; zeros(chunk,1)];
         yout = [yout; zeros(chunk,length(y))];
      end
      tout(k) = t;
      yout(k,:) = y';

    dographics_3D(y); % update the graphics

    % get new hf1, hf1dot so we know what it is precisely
    junk = yderivs_3D(t,y);
    hf1=junk(11);
    hfdot1 = junk(12);

   end; % delta <= tau loop

        % Update the step size

     if delta ~= 0.0
      h = min(hmax, 0.8*h*(tau/delta)^pow);
     end % delta ~= 0.0 loop

  end; %(hf1+h*hfdot1>0) & t < tfinal loop

horig=h; % store the original time step because we are about to make
         % it really small in the next loop.

%%%%%%%%%%%%%% Nearing Heelstrike %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Now we are less than h `seconds' away from heelstrike.
% Do Newton's method to get to kneestrike
% or Poincare's method or whatever it is.
% Assume we want to find the right h to get us right to heelstrike
% swing foot is at height junk(9) moving with vertical
% velocity junk(10). Calculate h and iterate to convergence.

 while abs(hf1) > (tol/1000) & t<tfinal  % do this loop until foot
                                         % height is much less than tol.
  h=(-hf1/hfdot1);  % predicted h

   junk = yderivs_3D(t,y);  % get the slopes from the deriv file
   f(:,1) = junk(1:10); 
   for j = 1:5
    junk = yderivs_3D(t,y+h*f*beta(:,j));
    f(:,j+1) = junk(1:10);
   end % j = 1:5 loop

     % Assume error is aceptable since h is small, update the solution
       t = t + h;
       incr = h*f*gamma(:,1);
       y = y + incr;

   % get new hf1, hf1dot so we know what it is precisely
   junk = yderivs_3D(t,y);
   hf1=junk(11);
   hfdot1=junk(12);

 end;  % abs(hf1) > (tol/1000) & t<tfinal  loop
 % Now we are exactly at the pre-heelstrike state

 k = k+1;

    if k > length(tout)
        tout = [tout; zeros(chunk,1)];
        yout = [yout; zeros(chunk,length(y))];
    end % k > length(tout) 
 tout(k) = t;
 yout(k,:) = y.';

dographics_3D(y); % update the graphics

%%%%%%%%%%%%%%%%%%%%%%%%%%%% Heelstrike %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if t<tfinal
 y % display y
 disp('heelstrike')
 ynew = heelstrike_3D(y);  % call the heelstrike routine to calculate
                           % new angular rates and swap L-R parameters.
 hf1 = 0;  % reset foot height
 hfdot1=1; % reset foot velocity to some bogus temporary thing
 y = ynew' % set the state variable to whatever ynew is

 h=horig/3;  % reset time step h to something reasonable
  %store new y at same t
	 k = k+1;
         if k > length(tout)
             tout = [tout; zeros(chunk,1)];
             yout = [yout; zeros(chunk,length(y))];
         end % k > length(tout)
         tout(k) = t;
         yout(k,:) = y.';
 w=w+1;  % increment the step number
 t  % display the time
 step(w,:)=ynew;  % store the new step state 
end; % t<tfinal
%%%%%%%%%%%%%%%%%%%%%%%%% End Main Loop %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end;  % mail loop

tout = tout(1:k);   % don't know what this does exactly.
yout = yout(1:k,:); % but it was in MATLAB's code.
