% it_3D_mcg.m file for finding fixed points and eigenvalues
% of the stride function for ONE STEP (L -> R, not L -> R -> L).

% All code by Mariano Garcia (msg5@cornell.edu), 1998
% Some documentation of sorts and accompanying figures can
% be found in chaps 2 and 5 of my thesis, which can be downloaded from
% http://tam.cornell.edu/students/garcia/msghomepage.html
% or
% http://www.ccmr.cornell.edu/~ruinalab/
% also read the README.txt file


% This is a crude one-parameter searching algorithm. It does
% not know how to deal with bifurcations or turning points in
% parameter space.
% Currently set up to vary the slope angle but can be changed
% to vary any parameter. 
% Some extra background info in my thesis, chapter 2.
% Assumes that we start at or near a fixed point!

global g gam m R p23 p34 pc3 pc4 p4f Icm3 Icm4 h hspr hoff hdamp


tfinal = 10;  % make tfinal something larger than one step period so
              % it will not interfere with integration of one step
              % This may be obsolete with all the debugging I have done by now.

gamfinal=1;  % the number of fixed points that you want to find

for slope = 1:gamfinal % the loop to find fixed points as functions of a parameter
incr=0.001;  % parameter increment size

%gam=0.07+(slope-1)*incr;  % step through the different parameter values.
                          % Note that some parameter values will change other
                          % parameter values and so this line may be much
                          % more complicated. For instance, changing the
                          % foot radius while keeping the total walker length
                          % at 1 requires also resetting the foot-cm-to-hip
                          % length and cm position. Changing mass positions
                          % will change the total walker cm and inertia matrices.
                          % So one must have a care here.

% Apply Newton- Raphson method to locate fixed point of map (limit cycle)
% iterate forward one step to get new y at start of next step
% then g= ynew-yold, g=0 for limit cycle
% this file perturbs all of the components of y

zeta=sqrt(tol);  % The size of the perturbation to be applied
dg=zeros(8,8);   % clear space for matrices
dgdy=zeros(8,8);
n=0;
iterations=50;  % number of iterations after which we give up
fixedpoint='false';
% ndamper should be greater than 0.2!
ndamper=1;  % if we want to damp Newton's method, never found it to help much
            % while doing this.
s=0; % don't know what the hell s is for.
step='step ';
OK = ' OK';

while (strcmp(fixedpoint,'true')==0) & (n < iterations) % do this until you
                                     % find a fp or its clear that you won't

y=int_3D_step_mcg(t0,tfinal,y0,tol);  % evaluate stride function at fp
gold(1)=y(1)+y0(1);    fold(1)=y(1);  % set up function that we want to zero.
gold(2)=y(2)+y0(2)-pi; fold(2)=y(2);  % in this case, we look for a reflection
gold(3)=y(3)-y0(3);    fold(3)=y(3);  % of heading and bank angles/rates, and
gold(4)=y(4)-y0(4);    fold(4)=y(4);  % the same stance and swing pitch angles/rates
gold(5)=y(5)+y0(5);    fold(5)=y(5);  % at the end of the step as when we started.
gold(6)=y(6)+y0(6);    fold(6)=y(6);
gold(7)=y(7)-y0(7);    fold(7)=y(7);
gold(8)=y(8)-y0(8);    fold(8)=y(8);
disp('step 0 OK')

% now perturb each initial condition by a small amount zeta
% to pick off the corresponding column of the gradient matrix

for i=1:8
 yg = y0;
 yg(i) = y0(i)+zeta;
 y=int_3D_step_mcg(t0,tfinal,yg,tol);

 gnew(1)=y(1)+yg(1);      fnew(1)=y(1);
 gnew(2)=y(2)+yg(2)-pi;   fnew(2)=y(2);
 gnew(3)=y(3)-yg(3);      fnew(3)=y(3);
 gnew(4)=y(4)-yg(4);      fnew(4)=y(4);
 gnew(5)=y(5)+yg(5);      fnew(5)=y(5);
 gnew(6)=y(6)+yg(6);      fnew(6)=y(6);
 gnew(7)=y(7)-yg(7);      fnew(7)=y(7);
 gnew(8)=y(8)-yg(8);      fnew(8)=y(8);

 dg(:,i)=(gnew-gold)';  % this is for getting the jacobian of the difference function
 df(:,i)=(fnew-fold)';  % this is for getting the jacobian of the stride function

 number=num2str(i);
 disp([step number OK])  % display where you are so the operator doesn't wonder
                        % what's going on (at least no more than usual).

end; % i=1:8 loop

dgdy=(1/zeta).*dg;  % This is the jacobian of the difference function

cond(dgdy)  % Display condition number, if you are interested

dely=dgdy\(-gold)';  % compute a delta y to find a fixed point
dely=ndamper.*dely   % and damp it


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (norm(gold) < tol)  %  if the difference function < tolerance then we have a fp
 fixedpoint='true'
end;

% increment y0
y0(1) = y0(1)+dely(1);
y0(2) = y0(2)+dely(2);
y0(3) = y0(3)+dely(3);
y0(4) = y0(4)+dely(4); 
y0(5) = y0(5)+dely(5);
y0(6) = y0(6)+dely(6);
y0(7) = y0(7)+dely(7);
y0(8) = y0(8)+dely(8)


n=n+1;

end; % end of while (strcmp(fixedpoint,'true')==0) & (n < iterations) loop

 if n<iterations 

% can store whatever you want to save about this fixed point here

[junk,time]=check_3D_step_mcg(t0,tfinal,y0,tol);  % gets some info about
                                                  % the gait cycle

fp(slope,:)=y0;
cmx(slope) = pc3(1);
cmy(slope) = pc3(2);
cmz(slope) = pc3(3);
radius(slope) = R;
tilt(slope)=gam;
hip(slope)=h;
hdampc(slope)=hdamp;
hspringk(slope)=hspr;
sdampc(slope) = sdamp;
Izz(slope)=Icm3(3,3);
Ixx(slope)=Icm3(1,1);
Iyy(slope)=Icm3(2,2);
Ixy(slope)=Icm3(1,2);
Ixz(slope)=Icm3(1,3);
Iyz(slope)=Icm3(2,3);

steptime(slope)=time;


J1=(1/zeta).*df;  % two ways to get the jacobian matrix, I think
                  % they give identical results

% or can also do
newI =[ -1     0     0     0     0     0     0   0
         0    -1     0     0     0     0     0   0
         0     0     1     0     0     0     0   0
         0     0     0     1     0     0     0   0
         0     0     0     0    -1     0     0   0
         0     0     0     0     0    -1     0   0
         0     0     0     0     0     0     1   0
         0     0     0     0     0     0     0   1];
J2=dgdy+newI;
%eig(J2)'
loc(slope,:)=eig(J1)'

 % find the maximum eigenvalue
 maxval(slope) = 0;

 for cc=1:8
  if abs(loc(slope,cc)) > maxval(slope)
   maxval(slope) = abs(loc(slope,cc));
  end; % if abs(loc(slope,cc)) > maxval(slope)
 end; % cc=1:8

save 3D_mcg_data % store the info you found so far

end;  % n<iterations

end; % main loop
